<?php

namespace Drp\SimpleCms\Test\Feature;

use Drp\SimpleCms\ContentManager;
use Drp\SimpleCms\Contracts\ContentFactory;
use Drp\SimpleCms\Contracts\Publishable;
use Drp\SimpleCms\Events\ContentForReviewWasRejected;
use Drp\SimpleCms\Events\ContentWasPublished;
use Drp\SimpleCms\Events\ContentWasSentForReview;
use Drp\SimpleCms\Events\ContentWasUnPublished;
use Drp\SimpleCms\Test\Support\Fakes\OtherContent;
use Drp\SimpleCms\Test\Support\Fakes\TestContent;
use Drp\SimpleCms\Test\Support\Helpers\InteractsWithPublishing;
use Drp\SimpleCms\Test\Support\Helpers\InteractsWithUsers;
use Drp\SimpleCms\Test\TestCase;
use Illuminate\Auth\Access\AuthorizationException;
use Illuminate\Http\Request;
use Illuminate\Http\UploadedFile;

class ContentManagementTest extends TestCase
{
    use InteractsWithPublishing, InteractsWithUsers;

    /**
     * @test
     */
    public function I_can_access_the_content_manager_from_the_listing()
    {
        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create();

        $this
            ->actingAsAdmin()
            ->visitRoute('admin.content.index', 'test-content')
            ->seeLink(
                $content->title,
                route('admin.content.manage', $content->getContentRouteParameters()))
            ->click($content->title)
            ->seeRouteIs('admin.content.manage', $content->getContentRouteParameters());
    }

    /**
     * @test
     */
    public function I_can_create_a_new_content()
    {
        $this->disableExceptionHandler();
        $this
            ->actingAsAdmin()
            ->visitRoute('admin.content.index', 'test-content')
            ->type('My mindset', 'title')
            ->press(trans('simpleCms::admin.store', ['label' => 'test content label']))
            ->seeRouteIs('admin.content.manage', [
                'type' => 'test-content',
                'contentId' => TestContent::first()->id
            ])
            ->seeSuccessAlert(trans('simpleCms::admin.created', ['label' => 'Test content label']))
            ->seeInDatabase('test_contents', [
                'title' => 'My mindset',
                'status' => Publishable::STATUS_DRAFT
            ]);
    }


    /**
     * @test
     */
    public function when_the_content_is_updated_a_draft_version_is_created()
    {
        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create(['title' => 'Original title']);

        $this
            ->actingAsAdmin()
            ->visitRoute('admin.content.manage', $content->getContentRouteParameters())
            ->seeInField('title', $content->title)
            ->type('Updated title', 'title')
            ->press(trans('simpleCms::admin.update', ['label' => 'content']))
            ->seeRouteIs('admin.content.manage', $content->getContentRouteParameters())
            ->seeSuccessAlert(trans('simpleCms::admin.updated', ['label' => 'Test content label']))
            ->seeInDatabase('test_contents', [
                'content_id' => 1,
                'title' => 'Original title',
                'status' => Publishable::STATUS_PUBLISHED
            ])
            ->seeInDatabase('test_contents', [
                'content_id' => 1,
                'title' => 'Updated title',
                'status' => Publishable::STATUS_DRAFT
            ]);
    }

    /**
     * @test
     */
    public function input_manipulation_within_the_callback_is_persisted_to_the_content_manager()
    {
        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create();

        $content->requestCallback = function (Request $request) {
            $request->merge(['image' => 'image.jpg']);
        };

        app()->instance(ContentFactory::class, \Mockery::mock(ContentFactory::class, function ($mock) use ($content) {
            $mock
                ->shouldReceive('latestVersionByTypeAndContentId')
                ->with('test-content', 1)
                ->andReturn($content);
        }));

        $manager = \Mockery::spy(ContentManager::class);
        app()->instance(ContentManager::class, $manager);

        $this->actingAsAdmin()
            ->call(
                'PUT',
                route('admin.content.update', $content->getContentRouteParameters()),
                ['title' => 'Title',],
                [],
                ['image' => UploadedFile::fake()->image('avatar.jpg')]
            );

        $manager
            ->shouldHaveReceived('updateContent')
            ->with($content, ["title" => "Title", "image" => "image.jpg"]);
    }

    /**
     * @test
     */
    public function a_draft_is_only_created_when_the_content_has_been_updated()
    {
        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create();

        $this
            ->actingAsAdmin()
            ->visitRoute('admin.content.manage', $content->getContentRouteParameters())
            ->seeInField('title', $content->title)
            ->press(trans('simpleCms::admin.update', ['label' => 'content']))
            ->seeRouteIs('admin.content.manage', $content->getContentRouteParameters())
            ->seeInfoAlert(trans('simpleCms::content.no-change'))
            ->seeInDatabase('test_contents', [
                'content_id' => 1,
                'status' => Publishable::STATUS_PUBLISHED
            ])
            ->dontSeeInDatabase('test_contents', [
                'content_id' => 1,
                'status' => Publishable::STATUS_DRAFT
            ]);
    }


    /**
     * @test
     */
    public function when_the_content_is_in_draft_that_is_the_content_being_edited()
    {
        $publishedContent = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create();

        $draftContent = factory(TestContent::class)
            ->states(Publishable::STATUS_DRAFT)
            ->create(['content_id' => 1]);

        $this
            ->actingAsAdmin()
            ->visitRoute('admin.content.manage', $draftContent->getContentRouteParameters())
            ->assertAllDraftOptionsAreAvailable()
            ->seeInField('title', $draftContent->title)
            ->dontSeeInField('title', $publishedContent->title)
            ->seeLink(trans('simpleCms::content.original'))
            ->seeLink(trans('simpleCms::content.reset'));
    }

    /**
     * @test
     */
    public function I_can_send_new_content_for_review()
    {
        $this->expectsEvents(ContentWasSentForReview::class);

        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_DRAFT)
            ->create();

        $this
            ->actingAsAdmin()
            ->visitRoute('admin.content.manage', $content->getContentRouteParameters())
            ->seeLink(trans('simpleCms::content.review'), route('admin.content.review', $content->getVersionRouteParameters()))
            ->click(trans('simpleCms::content.review'))
            ->seeInDatabase('test_contents', [
                'content_id' => 1,
                'status' => Publishable::STATUS_REVIEW
            ])
            ->seeSuccessAlert(trans('simpleCms::content.review-sent'))
            ->seeText(Publishable::STATUS_REVIEW)
            ->seeLink(trans('simpleCms::content.reject'))
            ->dontSeeLink(trans('simpleCms::content.review'));
    }

    /**
     * @test
     */
    public function as_an_uploader_I_cannot_reject_content_in_review()
    {
        $this->disableExceptionHandler();

        try {

            $content = factory(TestContent::class)
                ->states(Publishable::STATUS_REVIEW)
                ->create();

            $this->actingAsUploader()
                ->visitRoute('admin.content.manage', $content->getContentRouteParameters())
                ->dontSeeLink(trans('simpleCms::content.reject'), route('admin.content.review', $content->getVersionRouteParameters()))
                ->visit(route('admin.content.publish', $content->getVersionRouteParameters()));
        } catch (AuthorizationException $exception) {
            return;
        }

        $this->fail('Uploader was allowed access');

    }

    /**
     * @test
     */
    public function as_an_admin_I_can_reject_a_content_in_review()
    {
        $this->expectsEvents(ContentForReviewWasRejected::class);

        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_REVIEW)
            ->create();

        $this
            ->actingAsAdmin()
            ->visitRoute('admin.content.manage', $content->getContentRouteParameters())
            ->seeLink(trans('simpleCms::content.reject'), route('admin.content.review', $content->getVersionRouteParameters()))
            ->click(trans('simpleCms::content.reject'))
            ->seeSuccessAlert(trans('simpleCms::content.review-rejected'));
    }

    /**
     * @test
     */
    public function as_an_admin_I_can_publish_the_content()
    {
        $this->expectsEvents(ContentWasPublished::class);

        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_DRAFT)
            ->create();

        $this
            ->actingAsAdmin()
            ->visitRoute('admin.content.manage', $content->getContentRouteParameters())
            ->seeLink(trans('simpleCms::content.publish-draft'), route('admin.content.publish', $content->getVersionRouteParameters()))
            ->click(trans('simpleCms::content.publish-draft'))
            ->seeInDatabase('test_contents', [
                'content_id' => 1,
                'status' => Publishable::STATUS_PUBLISHED
            ])
            ->seeSuccessAlert(trans('simpleCms::content.published'))
            ->dontSeeInElement('#publish-status', Publishable::STATUS_DRAFT)
            ->assertPublishedOptionsAreVisible()
            ->dontSeeElement('#draft-actions');
    }

    /**
     * @test
     */
    public function as_an_uploader_I_cannot_publish_the_content()
    {
        $this->disableExceptionHandler();

        try {

            $content = factory(TestContent::class)
                ->states(Publishable::STATUS_DRAFT)
                ->create();

            $this->actingAsUploader()
                ->visitRoute('admin.content.manage', $content->getContentRouteParameters())
                ->dontSeeLink(trans('simpleCms::content.publish'), route('admin.content.publish', $content->getVersionRouteParameters()))
                ->visit(route('admin.content.publish', $content->getVersionRouteParameters()));

        } catch (AuthorizationException $exception) {
            return;
        }

        $this->fail('Uploader was allowed access');
    }


    /**
     * @test
     */
    public function as_an_admin_I_can_unpublish_the_content()
    {
        $this->expectsEvents(ContentWasUnPublished::class);

        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create();

        $this
            ->actingAsAdmin()
            ->visitRoute('admin.content.manage', $content->getContentRouteParameters())
            ->seeLink(trans('simpleCms::content.unpublish'), route('admin.content.publish', $content->getVersionRouteParameters()))
            ->click(trans('simpleCms::content.unpublish'))
            ->seeInDatabase('test_contents', [
                'content_id' => 1,
                'status' => Publishable::STATUS_UNPUBLISHED
            ])
            ->seeSuccessAlert(trans('simpleCms::content.unpublished'))
            ->assertUnPublishedOptionsAreVisible()
            ->dontSeeElement('#draft-actions');
    }

    /**
     * @test
     */
    public function as_an_admin_I_can_publish_the_content_when_in_review()
    {
        factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create();

        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_REVIEW)
            ->create(['content_id' => 1]);

        $this
            ->actingAsAdmin()
            ->visitRoute('admin.content.manage', $content->getContentRouteParameters())
            ->seeLink(trans('simpleCms::content.publish-draft'), route('admin.content.publish', $content->getVersionRouteParameters()))
            ->click(trans('simpleCms::content.publish-draft'))
            ->seeInDatabase('test_contents', [
                'content_id' => 1,
                'status' => Publishable::STATUS_PUBLISHED
            ])
            ->seeSuccessAlert(trans('simpleCms::content.published'))
            ->dontSeeInElement('#publish-status', Publishable::STATUS_DRAFT)
            ->assertPublishedOptionsAreVisible()
            ->dontSeeElement('#draft-actions');
    }

    /**
     * @test
     */
    public function the_form_is_validated_when_it_is_being_published()
    {
        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_DRAFT)
            ->create(['title' => null]);

        $this
            ->actingAsAdmin()
            ->visitRoute('admin.content.manage', $content->getContentRouteParameters())
            ->click(trans('simpleCms::content.publish-draft'))
            ->seeValidationAlert();
    }

    /**
     * @test
     */
    public function the_content_is_validated_on_creation()
    {
        $this
            ->actingAsAdmin()
            ->visitRoute('admin.content.index', 'test-content')
            ->type('', 'title')
            ->press(trans('simpleCms::admin.store', ['label' => 'test content label']))
            ->seeRouteIs('admin.content.index', 'test-content')
            ->seeValidationAlert();
    }

    /**
     * @test
     */
    public function the_content_is_validated_on_updating()
    {
        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create();

        $this
            ->visitRoute('admin.content.manage', $content->getContentRouteParameters())
            ->submitForm(trans('simpleCms::admin.update', ['label' => 'content']), ['title' => ''])
            ->seeRouteIs('admin.content.manage', $content->getContentRouteParameters())
            ->seeValidationAlert();
    }

    /**
     * @test
     */
    public function as_an_admin_I_can_reset_a_draft_to_the_original_content()
    {
        factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create();

        $draft = factory(TestContent::class)
            ->states(Publishable::STATUS_DRAFT)
            ->create(['title' => null, 'content_id' => 1]);

        $this
            ->actingAsAdmin()
            ->visitRoute('admin.content.manage', $draft->getContentRouteParameters())
            ->assertAllDraftOptionsAreAvailable()
            ->click(trans('simpleCms::content.reset'))
            ->dontSeeInDatabase('test_contents', [
                'content_id' => $draft->id,
                'status' => Publishable::STATUS_DRAFT
            ])
            ->seeSuccessAlert(trans('simpleCms::content.was-reset'))
            ->dontSeeInElement('#publish-status', Publishable::STATUS_DRAFT);
    }

    /**
     * @test
     */
    public function a_content_cannot_be_reset_if_it_is_the_only_version()
    {
        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_DRAFT)
            ->create();

        $this
            ->actingAsAdmin()
            ->visitRoute('admin.content.manage', $content->getContentRouteParameters())
            ->assertInitialDraftOptionsAreAvailable()
            ->dontSeeLink(trans('simpleCms::content.reset'), route('admin.content.reset', $content->getVersionRouteParameters()))
            ->get(route('admin.content.reset', $content->getVersionRouteParameters()))
            ->followRedirects()
            ->seeErrorAlert(trans('simpleCms::content.not-reset'));
    }

    /**
     * @test
     */
    public function a_content_cannot_be_reset_if_it_has_no_draft()
    {
        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create();

        $this
            ->actingAsAdmin()
            ->visitRoute('admin.content.manage', $content->getContentRouteParameters())
            ->dontSeeLink(trans('simpleCms::content.reset'), route('admin.content.reset', $content->getVersionRouteParameters()))
            ->get(route('admin.content.reset', $content->getVersionRouteParameters()))
            ->followRedirects()
            ->seeErrorAlert(trans('simpleCms::content.not-reset'));
    }

    /**
     * @test
     */
    public function as_an_uploader_user_I_cannot_reset_a_draft_to_the_original_content()
    {

        $this->disableExceptionHandler();

        try {

            $content = factory(TestContent::class)
                ->states(Publishable::STATUS_DRAFT)
                ->create();

            $this->actingAsUploader()
                ->visitRoute('admin.content.manage', $content->getContentRouteParameters())
                ->dontSeeLink(trans('simpleCms::content.reset'), route('admin.content.reset', $content->getVersionRouteParameters()))
                ->visit(route('admin.content.reset', $content->getVersionRouteParameters()));

        } catch (AuthorizationException $exception) {
            return;
        }

        $this->fail('Uploader was allowed access');
    }

    /**
     * @test
     */
    public function as_an_admin_I_can_delete_an_individual_content()
    {
        $content = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create();

        $content2 = factory(TestContent::class)->states(Publishable::STATUS_DRAFT)
            ->create([
                'content_id' => 1
            ]);

        $this->actingAsAdmin()
            ->visitRoute('admin.content.manage', $content->getContentRouteParameters())
            ->seeLink(trans('simpleCms::admin.delete', ['label' => 'content']), route('admin.content.destroy', $content->getContentRouteParameters()))
            ->call('delete', route('admin.content.destroy', $content->getContentRouteParameters()));

        $this->assertRedirectedTo(route('admin.content.index', ['type' => 'test-content']))
            ->followRedirects()
            ->seeSuccessAlert(trans('simpleCms::admin.deleted', ['label' => 'Content']))
            ->dontSeeInDatabase('test_contents', ['content_id' => $content->id])
            ->dontSeeInDatabase('test_contents', ['content_id' => $content2->id]);
    }

    /**
     * @test
     */
    public function as_an_uploader_I_can_not_delete_an_individual_content()
    {

        $this->disableExceptionHandler();

        try {

            $content = factory(TestContent::class)
                ->states(Publishable::STATUS_DRAFT)
                ->create();

            $this->actingAsUploader()
                ->visitRoute('admin.content.manage', $content->getContentRouteParameters())
                ->dontSeeText('Delete')
                ->dontSeeInElement('button', trans('simpleCms::admin.delete', ['label' => 'content']))
                ->call('delete', route('admin.content.destroy', $content->getContentRouteParameters()));


        } catch (AuthorizationException $exception) {
            $this->seeInDatabase('test_contents', ['content_id' => $content->id]);
            return;
        }

        $this->fail('Uploader was allowed access');


    }

    /**
     * @test
     */
    public function I_can_create_a_second_content()
    {
        factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create();

        $newStudy = factory(TestContent::class)
            ->states(Publishable::STATUS_DRAFT)
            ->create();


        $this
            ->actingAsAdmin()
            ->visitRoute('admin.content.manage', $newStudy->getContentRouteParameters())
            ->click(trans('simpleCms::content.publish-draft'))
            ->seeInDatabase('test_contents', [
                'content_id' => 1,
                'status' => Publishable::STATUS_PUBLISHED
            ])
            ->seeInDatabase('test_contents', [
                'content_id' => 2,
                'status' => Publishable::STATUS_PUBLISHED
            ]);
    }

    /**
     * @test
     */
    public function if_a_content_has_a_draft_its_published_is_not_shown_in_the_listing()
    {
        $publishedStudy = factory(TestContent::class)
            ->states(Publishable::STATUS_PUBLISHED)
            ->create();

        $draftStudy = factory(TestContent::class)
            ->states(Publishable::STATUS_DRAFT)
            ->create([
                'content_id' => 1,
                'title' => 'updated'
            ]);

        $this
            ->actingAsAdmin()
            ->visitRoute('admin.content.index', [
                'type' => 'test-content',
            ])
            ->seeText($draftStudy->title)
            ->dontSeeText($publishedStudy->title);
    }


    /**
     * @test
     */
    public function I_view_a_list_of_studies_that_need_reviewing()
    {
        $this->disableExceptionHandler();;

        $content1 = TestContent::create([
            'title' => 'My content',
            'status' => Publishable::STATUS_REVIEW
        ]);

        $content2 = OtherContent::create([
            'title' => 'Your content',
            'status' => Publishable::STATUS_REVIEW
        ]);

        $content3 = TestContent::create([
            'title' => 'Our content',
            'status' => Publishable::STATUS_PUBLISHED
        ]);


        $this->actingAsAdmin()
            ->visitRoute('admin.content.review.index')
            ->seeText('Test content label')
            ->seeLink(
                'My content',
                route('admin.content.manage', $content1->getContentRouteParameters())
            )
            ->seeText('Other content label')
            ->seeLink(
                'Your content',
                route('admin.content.manage', $content2->getContentRouteParameters())
            )
            ->dontSeeLink(
                'Your content',
                route('admin.content.manage', $content3->getContentRouteParameters())
            );
    }

}
