<?php

namespace Drp\SimpleCms\Test;

use Drp\ProjectHelpers\Tests\Concerns\InteractsWithExceptions;
use Drp\ProjectHelpers\Tests\Concerns\InteractsWithModels;
use Drp\ProjectHelpers\Tests\Concerns\InteractsWithAlerts;
use Drp\SimpleCms\Policies\PublishablePolicy;
use Drp\SimpleCms\SimpleCmsServiceProvider;
use Drp\SimpleCms\Test\Support\Fakes\OtherContent;
use Drp\SimpleCms\Test\Support\Fakes\TestContent;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Gate;
use Illuminate\Support\Facades\Route;
use Orchestra\Testbench\BrowserKit\TestCase as Orchestra;

/**
 * Class TestCase
 */
class TestCase extends Orchestra
{

    use InteractsWithModels, InteractsWithAlerts, InteractsWithExceptions;

    /** @var \Drp\SimpleCms\Test\TestHelper */
    protected $testHelper;

    /**
     *
     */
    public function setUp(): void
    {
        $this->testHelper = new TestHelper();

        parent::setUp();

        $this->setUpDatabase($this->app);

        config(['simple_cms.content_types' => [
            'test-content' => TestContent::class,
            'other-content' => OtherContent::class,
        ]]);

        $this->registerPolicies();
        $this->registerTestRoutes();
    }

    /**
     * Register the application's policies.
     *
     * @return void
     */
    public function registerPolicies()
    {
        foreach (config('simple_cms.content_types') as $class) {
            Gate::policy($class, PublishablePolicy::class);
        }
    }

    /**
     * @param \Illuminate\Foundation\Application $app
     *
     * @return array
     */
    protected function getPackageProviders($app)
    {
        return [
            SimpleCmsServiceProvider::class
        ];
    }

    /**
     * @param \Illuminate\Foundation\Application $app
     */
    protected function getEnvironmentSetUp($app)
    {
        $this->testHelper->initializeTempDirectory();
        $app['config']->set('database.default', 'sqlite');
        $app['config']->set('mail.driver', 'log');
        $app['config']->set('database.default', 'sqlite');
        $app['config']->set('database.connections.sqlite', [
            'driver' => 'sqlite',
            'database' => $this->testHelper->getTempDirectory() . '/database.sqlite',
            'prefix' => '',
        ]);
        $app['config']->set('filesystems.disks.local', [
            'driver' => 'local',
            'root' => $this->testHelper->getTempDirectory(),
        ]);
        $app['config']->set('filesystems.disks.secondLocal', [
            'driver' => 'local',
            'root' => $this->testHelper->getTempDirectory() . '/secondDisk',
        ]);

        $app['config']->set('view.paths', [base_path('resources/views'), $this->testHelper->getTestViews()]);
        $app['config']->set('app.key', '6rE9Nz59bGRbeMATftriyQjrpF7DcOQm');
    }


    /**
     * @param \Illuminate\Foundation\Application $app
     */
    protected function setUpDatabase($app)
    {
        file_put_contents($this->testHelper->getTempDirectory() . '/database.sqlite', null);

        $app['db']->connection()->getSchemaBuilder()->create('test_contents', function (Blueprint $table) {
            $table->increments('id');
            $table->string('title')->nullable();
            $table->string('status')->nullable();
            $table->string('content')->nullable();
            $table->integer('content_id')->nullable();
            $table->unique(['content_id', 'status']);
            $table->timestamps();
        });

        $app['db']->connection()->getSchemaBuilder()->create('other_contents', function (Blueprint $table) {
            $table->increments('id');
            $table->string('title')->nullable();
            $table->string('status')->nullable();
            $table->integer('content_id')->nullable();
            $table->unique(['content_id', 'status']);
            $table->timestamps();
        });


        $this->withFactories(__DIR__ . '/Support/factories');
    }

    protected function registerTestRoutes()
    {
        Route::get('/home', [
            'as' => 'home',
            'middleware' => ['previewMode', 'published'],
            'uses' => function () {
                return view('show', ['testContent' => null]);
            }
        ]);

        Route::get('/test-content', [
            'middleware' => ['previewMode', 'published'],
            'as' => 'test-content.index',
            'uses' => function () {
                return view('index', ['testContents' => TestContent::all()]);
            }
        ]);

        Route::get('/test-content/{title}', [
            'middleware' => ['previewMode', 'published'],
            'as' => 'test-content',
            'uses' => function ($title) {
                return view('show', ['testContent' => TestContent::where('title', $title)->first()]);
            }
        ]);

        Route::get('/admin', [
            'as' => 'admin.dashboard',
            'uses' => function () {
                return view('admin');
            }
        ]);
    }

}
